'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _class, _temp;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _AMapContext = require('../AMapContext');

var _AMapContext2 = _interopRequireDefault(_AMapContext);

var _breakIfNotChildOfAMap = require('../utils/breakIfNotChildOfAMap');

var _breakIfNotChildOfAMap2 = _interopRequireDefault(_breakIfNotChildOfAMap);

var _cloneDeep = require('../utils/cloneDeep');

var _cloneDeep2 = _interopRequireDefault(_cloneDeep);

var _createEventCallback = require('../utils/createEventCallback');

var _createEventCallback2 = _interopRequireDefault(_createEventCallback);

var _isShallowEqual = require('../utils/isShallowEqual');

var _isShallowEqual2 = _interopRequireDefault(_isShallowEqual);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/**
 * Fields that need to be deep copied.
 * AMap LBS library mutates options. Deep copy those options before passing to AMap so that
 * props won't be mutated.
 */
var NEED_DEEP_COPY_FIELDS = ['center'];

/**
 * Circle binding.
 * Circle has the same options as AMap.Circle unless highlighted below.
 * {@link https://lbs.amap.com/api/javascript-api/reference/overlay#circle}
 */
var Circle = (_temp = _class = function (_React$Component) {
  _inherits(Circle, _React$Component);

  _createClass(Circle, null, [{
    key: 'parseCircleOptions',


    /**
     * Parse AMap.Circle options.
     * Named properties are event callbacks, other properties are circle options.
     */
    value: function parseCircleOptions(props) {
      var onComplete = props.onComplete,
          onClick = props.onClick,
          onDblClick = props.onDblClick,
          onRightClick = props.onRightClick,
          onHide = props.onHide,
          onShow = props.onShow,
          onMouseDown = props.onMouseDown,
          onMouseUp = props.onMouseUp,
          onMouseOver = props.onMouseOver,
          onMouseOut = props.onMouseOut,
          onChange = props.onChange,
          onTouchStart = props.onTouchStart,
          onTouchMove = props.onTouchMove,
          onTouchEnd = props.onTouchEnd,
          circleOptions = _objectWithoutProperties(props, ['onComplete', 'onClick', 'onDblClick', 'onRightClick', 'onHide', 'onShow', 'onMouseDown', 'onMouseUp', 'onMouseOver', 'onMouseOut', 'onChange', 'onTouchStart', 'onTouchMove', 'onTouchEnd']);

      return circleOptions;
    }

    /**
     * Define event name mapping relations of react binding Circle and AMap.Circle.
     * Initialise AMap.Circle and bind events.
     * Fire complete action as soon as bezier curve has been created.
     */


    /**
     * AMap map instance.
     */

  }]);

  function Circle(props, context) {
    _classCallCheck(this, Circle);

    var _this = _possibleConstructorReturn(this, (Circle.__proto__ || Object.getPrototypeOf(Circle)).call(this, props));

    var onComplete = _this.props.onComplete;


    var map = context;

    (0, _breakIfNotChildOfAMap2.default)('Circle', map);

    _this.circleOptions = Circle.parseCircleOptions(_this.props);

    _this.circle = _this.initCircle(map);

    _this.bindEvents();

    typeof onComplete === 'function' && onComplete(map, _this.circle);
    return _this;
  }

  /**
   * Update this.circle by calling AMap.Circle methods.
   */


  _createClass(Circle, [{
    key: 'shouldComponentUpdate',
    value: function shouldComponentUpdate(nextProps) {
      var nextCircleOptions = Circle.parseCircleOptions(nextProps);

      var newCircleOptions = (0, _cloneDeep2.default)(nextCircleOptions, NEED_DEEP_COPY_FIELDS);

      this.toggleVisible(this.circleOptions.visible, nextCircleOptions.visible);
      /**
       * Instead of calling one API for a specific option change, AMap.Circle exposes
       * a master method: setOptions, which will update every options with a single function call.
       */
      this.updateCircleWithAPI('setOptions', this.circleOptions, nextCircleOptions, newCircleOptions);

      this.circleOptions = nextCircleOptions;

      return false;
    }

    /**
     * Remove event listeners.
     * Destroy circle instance.
     */

  }, {
    key: 'componentWillUnmount',
    value: function componentWillUnmount() {
      this.AMapEventListeners.forEach(function (listener) {
        window.AMap.event.removeListener(listener);
      });

      this.circle.setMap(null);
      this.circle = null;
    }

    /**
     * Bind all events on map instance, and save event listeners which will be removed in
     * componentWillUnmount lifecycle.
     */

  }, {
    key: 'bindEvents',
    value: function bindEvents() {
      var _this2 = this;

      this.AMapEventListeners = [];

      /**
       * Construct event callbacks.
       */
      var eventCallbacks = this.parseEvents();

      Object.keys(eventCallbacks).forEach(function (key) {
        var eventName = key.substring(2).toLowerCase();
        var handler = eventCallbacks[key];

        _this2.AMapEventListeners.push(window.AMap.event.addListener(_this2.circle, eventName, handler));
      });
    }

    /**
     * Initialise AMap.Circle.
     */

  }, {
    key: 'initCircle',
    value: function initCircle(map) {
      var visible = this.props.visible;


      var newCircleOptions = (0, _cloneDeep2.default)(this.circleOptions, NEED_DEEP_COPY_FIELDS);

      var circle = new window.AMap.Circle(newCircleOptions);

      circle.setMap(map);

      if (visible === false) circle.hide();

      return circle;
    }

    /**
     * Return an object of all supported event callbacks.
     */

  }, {
    key: 'parseEvents',
    value: function parseEvents() {
      return {
        onClick: (0, _createEventCallback2.default)('onClick', this.circle).bind(this),
        onDblClick: (0, _createEventCallback2.default)('onDblClick', this.circle).bind(this),
        onRightClick: (0, _createEventCallback2.default)('onRightClick', this.circle).bind(this),
        onHide: (0, _createEventCallback2.default)('onHide', this.circle).bind(this),
        onShow: (0, _createEventCallback2.default)('onShow', this.circle).bind(this),
        onMouseOver: (0, _createEventCallback2.default)('onMouseOver', this.circle).bind(this),
        onMouseOut: (0, _createEventCallback2.default)('onMouseOut', this.circle).bind(this),
        onMouseDown: (0, _createEventCallback2.default)('onMouseDown', this.circle).bind(this),
        onMouseUp: (0, _createEventCallback2.default)('onMouseUp', this.circle).bind(this),
        onChange: (0, _createEventCallback2.default)('onChange', this.circle).bind(this),
        onTouchStart: (0, _createEventCallback2.default)('onTouchStart', this.circle).bind(this),
        onTouchMove: (0, _createEventCallback2.default)('onTouchMove', this.circle).bind(this),
        onTouchEnd: (0, _createEventCallback2.default)('onTouchEnd', this.circle).bind(this)
      };
    }

    /**
     * Hide or show circle.
     */

  }, {
    key: 'toggleVisible',
    value: function toggleVisible(previousProp, nextProp) {
      if (!(0, _isShallowEqual2.default)(previousProp, nextProp)) {
        if (nextProp === true) this.circle.show();
        if (nextProp === false) this.circle.hide();
      }
    }

    /**
     * Update AMap.Circle instance with named API.
     * Won't call API if prop does not change.
     */

  }, {
    key: 'updateCircleWithAPI',
    value: function updateCircleWithAPI(apiName, previousProp, nextProp, newProp) {
      if (!(0, _isShallowEqual2.default)(previousProp, nextProp)) {
        this.circle[apiName](newProp);
      }
    }

    /**
     * Render nothing.
     */

  }, {
    key: 'render',
    value: function render() {
      return null;
    }
  }]);

  return Circle;
}(_react2.default.Component), _class.propTypes = {
  /**
   * Show Circle by default, you can toggle show or hide by changing visible.
   */
  visible: _propTypes2.default.bool,
  /**
   * Event callback.
   * Signature:
   * (circle, ...event) => void
   * circle: AMap.Circle instance.
   * event: AMap event.
   */
  /* eslint-disable react/sort-prop-types,react/no-unused-prop-types */
  onComplete: _propTypes2.default.func,
  onClick: _propTypes2.default.func,
  onDblClick: _propTypes2.default.func,
  onRightClick: _propTypes2.default.func,
  onHide: _propTypes2.default.func,
  onShow: _propTypes2.default.func,
  onMouseDown: _propTypes2.default.func,
  onMouseUp: _propTypes2.default.func,
  onMouseOver: _propTypes2.default.func,
  onMouseOut: _propTypes2.default.func,
  onChange: _propTypes2.default.func,
  onTouchStart: _propTypes2.default.func,
  onTouchMove: _propTypes2.default.func,
  onTouchEnd: _propTypes2.default.func
  /* eslint-enable */
}, _class.defaultProps = {
  visible: true
}, _class.contextType = _AMapContext2.default, _temp);
exports.default = Circle;