'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _class, _temp;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _AMapContext = require('../AMapContext');

var _AMapContext2 = _interopRequireDefault(_AMapContext);

var _breakIfNotChildOfAMap = require('../utils/breakIfNotChildOfAMap');

var _breakIfNotChildOfAMap2 = _interopRequireDefault(_breakIfNotChildOfAMap);

var _cloneDeep = require('../utils/cloneDeep');

var _cloneDeep2 = _interopRequireDefault(_cloneDeep);

var _createEventCallback = require('../utils/createEventCallback');

var _createEventCallback2 = _interopRequireDefault(_createEventCallback);

var _isShallowEqual = require('../utils/isShallowEqual');

var _isShallowEqual2 = _interopRequireDefault(_isShallowEqual);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/**
 * Fields that need to be deep copied.
 * AMap LBS library mutates options. Deep copy those options before passing to AMap so that
 * props won't be mutated.
 */
var NEED_DEEP_COPY_FIELDS = ['position'];

/**
 * InfoWindow binding.
 * InfoWindow has the same options as AMap.InfoWindow unless highlighted below.
 * {@link http://lbs.amap.com/api/javascript-api/reference/infowindow}
 */
var InfoWindow = (_temp = _class = function (_React$Component) {
  _inherits(InfoWindow, _React$Component);

  _createClass(InfoWindow, null, [{
    key: 'parseInfoWindowOptions',


    /**
     * Parse AMap.InfoWindow options.
     * Named properties are event callbacks, other properties are infoWindow options.
     */
    value: function parseInfoWindowOptions(props) {
      var onComplete = props.onComplete,
          onChange = props.onChange,
          onOpen = props.onOpen,
          onClose = props.onClose,
          infoWindowOptions = _objectWithoutProperties(props, ['onComplete', 'onChange', 'onOpen', 'onClose']);

      var offset = infoWindowOptions.offset,
          size = infoWindowOptions.size;


      return _extends({}, infoWindowOptions, {
        // Will transform an array of two numbers into a Pixel instance
        offset: function () {
          if (offset instanceof window.AMap.Pixel) {
            return offset;
          }

          if (offset instanceof Array) {
            return new (Function.prototype.bind.apply(window.AMap.Pixel, [null].concat(_toConsumableArray(offset))))();
          }

          return new window.AMap.Pixel(0, 0);
        }(),
        // Will transform an array of two numbers into a Size instance
        size: function () {
          if (size instanceof window.AMap.Size) {
            return size;
          }

          if (size instanceof Array) {
            return new (Function.prototype.bind.apply(window.AMap.Size, [null].concat(_toConsumableArray(size))))();
          }

          return null;
        }()
      });
    }

    /**
     * Define event name mapping relations of react binding InfoWindow and AMap.InfoWindow.
     * Initialise AMap.InfoWindow and bind events.
     * Fire complete action as soon as bezier curve has been created.
     */


    /**
     * AMap map instance.
     */

  }]);

  function InfoWindow(props, context) {
    _classCallCheck(this, InfoWindow);

    var _this = _possibleConstructorReturn(this, (InfoWindow.__proto__ || Object.getPrototypeOf(InfoWindow)).call(this, props));

    var onComplete = _this.props.onComplete;


    _this.map = context;

    (0, _breakIfNotChildOfAMap2.default)('InfoWindow', _this.map);

    _this.infoWindowOptions = InfoWindow.parseInfoWindowOptions(_this.props);

    _this.infoWindow = _this.initInfoWindow(_this.map);

    _this.bindEvents();

    typeof onComplete === 'function' && onComplete(_this.map, _this.infoWindow);
    return _this;
  }

  /**
   * Update this.infoWindow by calling AMap.InfoWindow methods.
   */


  _createClass(InfoWindow, [{
    key: 'shouldComponentUpdate',
    value: function shouldComponentUpdate(nextProps) {
      var nextInfoWindowOptions = InfoWindow.parseInfoWindowOptions(nextProps);

      var newInfoWindowOptions = (0, _cloneDeep2.default)(nextInfoWindowOptions, NEED_DEEP_COPY_FIELDS);

      this.toggleVisible(this.infoWindowOptions.visible, nextInfoWindowOptions.visible, nextInfoWindowOptions.position);

      this.updateInfoWindowWithAPI('setContent', this.infoWindowOptions.content, nextInfoWindowOptions.content, newInfoWindowOptions.content);

      this.updateInfoWindowWithAPI('setPosition', this.infoWindowOptions.position, nextInfoWindowOptions.position, newInfoWindowOptions.position);

      this.updateInfoWindowWithAPI('setAnchor', this.infoWindowOptions.anchor, nextInfoWindowOptions.anchor, newInfoWindowOptions.anchor);

      this.updateInfoWindowWithAPI('setSize', this.infoWindowOptions.size, nextInfoWindowOptions.size, newInfoWindowOptions.size);

      this.infoWindowOptions = nextInfoWindowOptions;

      return false;
    }

    /**
     * Remove event listeners.
     * Destroy infoWindow instance.
     */

  }, {
    key: 'componentWillUnmount',
    value: function componentWillUnmount() {
      this.AMapEventListeners.forEach(function (listener) {
        window.AMap.event.removeListener(listener);
      });

      this.infoWindow.setMap(null);
      this.infoWindow = null;
    }

    /**
     * Bind all events on map instance, and save event listeners which will be removed in
     * componentWillUnmount lifecycle.
     */

  }, {
    key: 'bindEvents',
    value: function bindEvents() {
      var _this2 = this;

      this.AMapEventListeners = [];

      /**
       * Construct event callbacks.
       */
      var eventCallbacks = this.parseEvents();

      Object.keys(eventCallbacks).forEach(function (key) {
        var eventName = key.substring(2).toLowerCase();
        var handler = eventCallbacks[key];

        _this2.AMapEventListeners.push(window.AMap.event.addListener(_this2.infoWindow, eventName, handler));
      });
    }

    /**
     * Initialise AMap.InfoWindow
     */

  }, {
    key: 'initInfoWindow',
    value: function initInfoWindow(map) {
      var _props = this.props,
          position = _props.position,
          visible = _props.visible;


      var newInfoWindowOptions = (0, _cloneDeep2.default)(this.infoWindowOptions, NEED_DEEP_COPY_FIELDS);

      var infoWindow = new window.AMap.InfoWindow(newInfoWindowOptions);

      infoWindow.setMap(map);

      if (visible === true) infoWindow.open(map, position);

      return infoWindow;
    }

    /**
     * Return an object of all supported event callbacks.
     */

  }, {
    key: 'parseEvents',
    value: function parseEvents() {
      return {
        onChange: (0, _createEventCallback2.default)('onChange', this.infoWindow).bind(this),
        onOpen: (0, _createEventCallback2.default)('onOpen', this.infoWindow).bind(this),
        onClose: (0, _createEventCallback2.default)('onClose', this.infoWindow).bind(this)
      };
    }

    /**
     * Hide or show infoWindow.
     */

  }, {
    key: 'toggleVisible',
    value: function toggleVisible(previousProp, nextProp, position) {
      if (!(0, _isShallowEqual2.default)(previousProp, nextProp)) {
        if (nextProp === true) {
          this.infoWindow.open(this.map, position);
        }
        if (nextProp === false) this.infoWindow.close();
      }
    }

    /**
     * Update AMap.Circle instance with named API.
     * Won't call API if prop does not change.
     */

  }, {
    key: 'updateInfoWindowWithAPI',
    value: function updateInfoWindowWithAPI(apiName, previousProp, nextProp, newProp) {
      if (!(0, _isShallowEqual2.default)(previousProp, nextProp)) {
        this.infoWindow[apiName](newProp);
      }
    }

    /**
     * Render nothing.
     */

  }, {
    key: 'render',
    value: function render() {
      return null;
    }
  }]);

  return InfoWindow;
}(_react2.default.Component), _class.propTypes = {
  /**
   * An array of two numbers or AMap.Pixel.
   */
  /* eslint-disable react/no-unused-prop-types */
  offset: _propTypes2.default.oneOfType([_propTypes2.default.arrayOf(_propTypes2.default.number), _propTypes2.default.object]),
  /**
   * An array of two numbers or AMap.Pixel.
   */
  position: _propTypes2.default.oneOfType([_propTypes2.default.arrayOf(_propTypes2.default.number), _propTypes2.default.object]).isRequired,
  /**
   * An array of two numbers, width and height or AMap.Size.
   */
  size: _propTypes2.default.oneOfType([_propTypes2.default.arrayOf(_propTypes2.default.number), _propTypes2.default.object]),
  /**
   * Show InfoWindow by default, you can toggle show or hide by changing visible.
   */
  visible: _propTypes2.default.bool,
  /* eslint-disable react/sort-prop-types,react/no-unused-prop-types */
  /**
   * Event callback.
   * Signature:
   * (infoWindow, ...event) => void
   * infoWindow: AMap.InfoWindow instance.
   * event: AMap event.
   */
  onComplete: _propTypes2.default.func,
  onChange: _propTypes2.default.func,
  onOpen: _propTypes2.default.func,
  onClose: _propTypes2.default.func
  /* eslint-enable */
}, _class.defaultProps = {
  visible: true
}, _class.contextType = _AMapContext2.default, _temp);
exports.default = InfoWindow;