'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _class, _temp;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _AMapContext = require('../AMapContext');

var _AMapContext2 = _interopRequireDefault(_AMapContext);

var _breakIfNotChildOfAMap = require('../utils/breakIfNotChildOfAMap');

var _breakIfNotChildOfAMap2 = _interopRequireDefault(_breakIfNotChildOfAMap);

var _cloneDeep = require('../utils/cloneDeep');

var _cloneDeep2 = _interopRequireDefault(_cloneDeep);

var _createEventCallback = require('../utils/createEventCallback');

var _createEventCallback2 = _interopRequireDefault(_createEventCallback);

var _isShallowEqual = require('../utils/isShallowEqual');

var _isShallowEqual2 = _interopRequireDefault(_isShallowEqual);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/**
 * Fields that need to be deep copied.
 * AMap LBS library mutates options. Deep copy those options before passing to AMap so that
 * props won't be mutated.
 */
var NEED_DEEP_COPY_FIELDS = ['data', 'style'];

/**
 * MassMarks binding.
 * MassMarks has the same options as AMap.MassMarks unless highlighted below.
 * {@link http://lbs.amap.com/api/javascript-api/reference/layer/#MassMarks}
 */
var MassMarks = (_temp = _class = function (_React$Component) {
  _inherits(MassMarks, _React$Component);

  _createClass(MassMarks, null, [{
    key: 'parseMassMarksOptions',


    /**
     * Parse AMap.MassMarks options.
     * Named properties are event callbacks, other properties are massMarks options.
     */
    value: function parseMassMarksOptions(props) {
      var onComplete = props.onComplete,
          onClick = props.onClick,
          onDblClick = props.onDblClick,
          onMouseOver = props.onMouseOver,
          onMouseOut = props.onMouseOut,
          onMouseUp = props.onMouseUp,
          onMouseDown = props.onMouseDown,
          onTouchStart = props.onTouchStart,
          onTouchEnd = props.onTouchEnd,
          massMarksOptions = _objectWithoutProperties(props, ['onComplete', 'onClick', 'onDblClick', 'onMouseOver', 'onMouseOut', 'onMouseUp', 'onMouseDown', 'onTouchStart', 'onTouchEnd']);

      var style = massMarksOptions.style;


      return _extends({}, massMarksOptions, {
        style: function () {
          if (style instanceof Array) {
            return style.map(function (styleObject) {
              return _extends({}, styleObject, {
                anchor: styleObject.anchor instanceof window.AMap.Pixel ? styleObject.anchor : new (Function.prototype.bind.apply(window.AMap.Pixel, [null].concat(_toConsumableArray(styleObject.anchor))))(),
                size: styleObject.size instanceof window.AMap.Size ? styleObject.size : new (Function.prototype.bind.apply(window.AMap.Size, [null].concat(_toConsumableArray(styleObject.size))))()
              });
            });
          }

          return _extends({}, style, {
            anchor: style.anchor instanceof window.AMap.Pixel ? style.anchor : new (Function.prototype.bind.apply(window.AMap.Pixel, [null].concat(_toConsumableArray(style.anchor))))(),
            size: style.size instanceof window.AMap.Size ? style.size : new (Function.prototype.bind.apply(window.AMap.Size, [null].concat(_toConsumableArray(style.size))))()
          });
        }()
      });
    }

    /**
     * Define event name mapping relations of react binding MassMarks and AMap.MassMarks.
     * Initialise AMap.MassMarks and bind events.
     */


    /**
     * AMap map instance.
     */

  }]);

  function MassMarks(props, context) {
    _classCallCheck(this, MassMarks);

    var _this = _possibleConstructorReturn(this, (MassMarks.__proto__ || Object.getPrototypeOf(MassMarks)).call(this, props));

    var map = context;

    (0, _breakIfNotChildOfAMap2.default)('MassMarks', map);

    _this.massMarksOptions = MassMarks.parseMassMarksOptions(_this.props);

    _this.massMarks = _this.initMassMarks(map);

    _this.bindEvents();
    return _this;
  }

  /**
   * Update this.massMarks by calling AMap.MassMarks methods
   */


  _createClass(MassMarks, [{
    key: 'shouldComponentUpdate',
    value: function shouldComponentUpdate(nextProps) {
      var nextMassMarksOptions = MassMarks.parseMassMarksOptions(nextProps);

      var newMassMarksOptions = (0, _cloneDeep2.default)(nextMassMarksOptions, NEED_DEEP_COPY_FIELDS);

      this.toggleVisible(this.massMarksOptions.visible, nextMassMarksOptions.visible);

      this.updateMassMarksWithAPI('setStyle', this.massMarksOptions.style, nextMassMarksOptions.style, newMassMarksOptions.style);

      this.updateMassMarksWithAPI('setData', this.massMarksOptions.data, nextMassMarksOptions.data, newMassMarksOptions.data);

      this.massMarksOptions = nextMassMarksOptions;

      return false;
    }

    /**
     * Remove event listeners.
     * Destroy massMarks instance.
     */

  }, {
    key: 'componentWillUnmount',
    value: function componentWillUnmount() {
      this.AMapEventListeners.forEach(function (listener) {
        window.AMap.event.removeListener(listener);
      });

      this.massMarks.setMap(null);
      this.massMarks = null;
    }

    /**
     * Bind all events on map instance, and save event listeners which will be removed in
     * componentWillUnmount lifecycle.
     */

  }, {
    key: 'bindEvents',
    value: function bindEvents() {
      var _this2 = this;

      this.AMapEventListeners = [];

      /**
       * Construct event callbacks.
       */
      var eventCallbacks = this.parseEvents();

      Object.keys(eventCallbacks).forEach(function (key) {
        var eventName = key.substring(2).toLowerCase();
        var handler = eventCallbacks[key];

        _this2.AMapEventListeners.push(window.AMap.event.addListener(_this2.massMarks, eventName, handler));
      });
    }

    /**
     * Initialise AMap massMarks layer.
     */

  }, {
    key: 'initMassMarks',
    value: function initMassMarks(map) {
      var _props = this.props,
          data = _props.data,
          visible = _props.visible;


      var newMassMarksData = (0, _cloneDeep2.default)(data);
      var newMassMarksOptions = (0, _cloneDeep2.default)(this.massMarksOptions, NEED_DEEP_COPY_FIELDS);

      var massMarks = new window.AMap.MassMarks(newMassMarksData, newMassMarksOptions);

      massMarks.setMap(map);

      if (visible === false) massMarks.hide();

      return massMarks;
    }

    /**
     * Return an object of all supported event callbacks.
     */

  }, {
    key: 'parseEvents',
    value: function parseEvents() {
      return {
        onComplete: (0, _createEventCallback2.default)('onComplete', this.massMarks).bind(this),
        onClick: (0, _createEventCallback2.default)('onClick', this.massMarks).bind(this),
        onDblClick: (0, _createEventCallback2.default)('onDblClick', this.massMarks).bind(this),
        onMouseOver: (0, _createEventCallback2.default)('onMouseOver', this.massMarks).bind(this),
        onMouseOut: (0, _createEventCallback2.default)('onMouseOut', this.massMarks).bind(this),
        onMouseUp: (0, _createEventCallback2.default)('onMouseUp', this.massMarks).bind(this),
        onMouseDown: (0, _createEventCallback2.default)('onMouseDown', this.massMarks).bind(this),
        onTouchStart: (0, _createEventCallback2.default)('onTouchStart', this.massMarks).bind(this),
        onTouchEnd: (0, _createEventCallback2.default)('onTouchEnd', this.massMarks).bind(this)
      };
    }

    /**
     * Hide or show massMarks.
     */

  }, {
    key: 'toggleVisible',
    value: function toggleVisible(previousProp, nextProp) {
      if (!(0, _isShallowEqual2.default)(previousProp, nextProp)) {
        if (nextProp === true) this.massMarks.show();
        if (nextProp === false) this.massMarks.hide();
      }
    }

    /**
     * Update AMap.MassMarks instance with named API and given value.
     * Won't call API if the given value does not change.
     */

  }, {
    key: 'updateMassMarksWithAPI',
    value: function updateMassMarksWithAPI(apiName, previousProp, nextProp, newProp) {
      if (!(0, _isShallowEqual2.default)(previousProp, nextProp)) {
        this.massMarks[apiName](newProp);
      }
    }

    /**
     * Render nothing.
     */

  }, {
    key: 'render',
    value: function render() {
      return null;
    }
  }]);

  return MassMarks;
}(_react2.default.Component), _class.propTypes = {
  /**
   * MassMarks dataset.
   */
  data: _propTypes2.default.arrayOf(_propTypes2.default.shape({
    lnglat: _propTypes2.default.arrayOf(_propTypes2.default.number).isRequired
  })).isRequired,
  /* eslint-disable react/no-unused-prop-types */
  style: _propTypes2.default.oneOfType([_propTypes2.default.arrayOf(_propTypes2.default.shape({
    /**
     * An array of two numbers or AMap.Pixel.
     */
    anchor: _propTypes2.default.oneOfType([_propTypes2.default.arrayOf(_propTypes2.default.number), _propTypes2.default.object]).isRequired,
    /**
     * An array of two numbers or AMap.Size.
     */
    size: _propTypes2.default.oneOfType([_propTypes2.default.arrayOf(_propTypes2.default.number), _propTypes2.default.object]).isRequired
  })), _propTypes2.default.shape({
    anchor: _propTypes2.default.oneOfType([_propTypes2.default.arrayOf(_propTypes2.default.number), _propTypes2.default.object]).isRequired,
    size: _propTypes2.default.oneOfType([_propTypes2.default.arrayOf(_propTypes2.default.number), _propTypes2.default.object]).isRequired
  })]).isRequired,
  /**
   * Show MassMarks by default, you can toggle show or hide by changing visible.
   */
  visible: _propTypes2.default.bool,
  /* eslint-disable react/sort-prop-types,react/no-unused-prop-types */
  /**
   * Event callback.
   * Signature:
   * (massMarks, ...event) => void
   * massMarks: AMap.MassMarks instance.
   * event: AMap event.
   */
  onComplete: _propTypes2.default.func,
  onClick: _propTypes2.default.func,
  onDblClick: _propTypes2.default.func,
  onMouseOver: _propTypes2.default.func,
  onMouseOut: _propTypes2.default.func,
  onMouseUp: _propTypes2.default.func,
  onMouseDown: _propTypes2.default.func,
  onTouchStart: _propTypes2.default.func,
  onTouchEnd: _propTypes2.default.func
  /* eslint-enable */
}, _class.defaultProps = {
  visible: true
}, _class.contextType = _AMapContext2.default, _temp);
exports.default = MassMarks;