'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _class, _temp;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _AMapContext = require('../AMapContext');

var _AMapContext2 = _interopRequireDefault(_AMapContext);

var _breakIfNotChildOfAMap = require('../utils/breakIfNotChildOfAMap');

var _breakIfNotChildOfAMap2 = _interopRequireDefault(_breakIfNotChildOfAMap);

var _cloneDeep = require('../utils/cloneDeep');

var _cloneDeep2 = _interopRequireDefault(_cloneDeep);

var _createEventCallback = require('../utils/createEventCallback');

var _createEventCallback2 = _interopRequireDefault(_createEventCallback);

var _isShallowEqual = require('../utils/isShallowEqual');

var _isShallowEqual2 = _interopRequireDefault(_isShallowEqual);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/**
 * Fields that need to be deep copied.
 * AMap LBS library mutates options. Deep copy those options before passing to AMap so that
 * props won't be mutated.
 */
var NEED_DEEP_COPY_FIELDS = ['path'];

/**
 * Polygon binding.
 * Polygon has the same options as AMap.Polygon unless highlighted below.
 * {@link http://lbs.amap.com/api/javascript-api/reference/overlay#polygon}
 */
var Polygon = (_temp = _class = function (_React$Component) {
  _inherits(Polygon, _React$Component);

  _createClass(Polygon, null, [{
    key: 'parsePolygonOptions',


    /**
     * Parse AMap.Polygon options.
     * Named properties are event callbacks, other properties are polygon options.
     */
    value: function parsePolygonOptions(props) {
      var onComplete = props.onComplete,
          onClick = props.onClick,
          onDblClick = props.onDblClick,
          onRightClick = props.onRightClick,
          onHide = props.onHide,
          onShow = props.onShow,
          onMouseDown = props.onMouseDown,
          onMouseUp = props.onMouseUp,
          onMouseOver = props.onMouseOver,
          onMouseOut = props.onMouseOut,
          onChange = props.onChange,
          onTouchStart = props.onTouchStart,
          onTouchMove = props.onTouchMove,
          onTouchEnd = props.onTouchEnd,
          polygonOptions = _objectWithoutProperties(props, ['onComplete', 'onClick', 'onDblClick', 'onRightClick', 'onHide', 'onShow', 'onMouseDown', 'onMouseUp', 'onMouseOver', 'onMouseOut', 'onChange', 'onTouchStart', 'onTouchMove', 'onTouchEnd']);

      return polygonOptions;
    }

    /**
     * Define event name mapping relations of react binding Polygon and AMap.Polygon.
     * Initialise AMap.Polygon and bind events.
     * Fire complete action as soon as polygon has been created.
     */


    /**
     * AMap map instance.
     */

  }]);

  function Polygon(props, context) {
    _classCallCheck(this, Polygon);

    var _this = _possibleConstructorReturn(this, (Polygon.__proto__ || Object.getPrototypeOf(Polygon)).call(this, props));

    var onComplete = props.onComplete;


    var map = context;

    (0, _breakIfNotChildOfAMap2.default)('Polygon', map);

    _this.polygonOptions = Polygon.parsePolygonOptions(_this.props);

    _this.polygon = _this.initPolygon(map);

    _this.bindEvents();

    typeof onComplete === 'function' && onComplete(map, _this.polygon);
    return _this;
  }

  /**
   * Update this.polygon by calling AMap.Polygon methods.
   */


  _createClass(Polygon, [{
    key: 'shouldComponentUpdate',
    value: function shouldComponentUpdate(nextProps) {
      var nextPolygonOptions = Polygon.parsePolygonOptions(nextProps);

      var newPolygonOptions = (0, _cloneDeep2.default)(nextPolygonOptions, NEED_DEEP_COPY_FIELDS);

      this.toggleVisible(this.polygonOptions.visible, nextPolygonOptions.visible);

      this.updatePolygonWithApi('setOptions', this.polygonOptions, nextPolygonOptions, newPolygonOptions);

      this.polygonOptions = nextPolygonOptions;

      return false;
    }

    /**
     * Remove event listeners.
     * Destroy polygon instance.
     */

  }, {
    key: 'componentWillUnmount',
    value: function componentWillUnmount() {
      this.AMapEventListeners.forEach(function (listener) {
        window.AMap.event.removeListener(listener);
      });

      this.polygon.setMap(null);
      this.polygon = null;
    }

    /**
     * Bind all events on map instance, and save event listeners which will be removed in
     * componentWillUnmount lifecycle.
     */

  }, {
    key: 'bindEvents',
    value: function bindEvents() {
      var _this2 = this;

      this.AMapEventListeners = [];

      /**
       * Construct event callbacks.
       */
      var eventCallbacks = this.parseEvents();

      Object.keys(eventCallbacks).forEach(function (key) {
        var eventName = key.substring(2).toLowerCase();
        var handler = eventCallbacks[key];

        _this2.AMapEventListeners.push(window.AMap.event.addListener(_this2.polygon, eventName, handler));
      });
    }

    /**
     * Initialise AMap polygon.
     */

  }, {
    key: 'initPolygon',
    value: function initPolygon(map) {
      var visible = this.props.visible;


      var newPolygonOptions = (0, _cloneDeep2.default)(this.polygonOptions, NEED_DEEP_COPY_FIELDS);

      var polygon = new window.AMap.Polygon(newPolygonOptions);

      polygon.setMap(map);

      if (visible === false) polygon.hide();

      return polygon;
    }

    /**
     * Return an object of all supported event callbacks.
     */

  }, {
    key: 'parseEvents',
    value: function parseEvents() {
      return {
        onClick: (0, _createEventCallback2.default)('onClick', this.polygon).bind(this),
        onDblClick: (0, _createEventCallback2.default)('onDblClick', this.polygon).bind(this),
        onRightClick: (0, _createEventCallback2.default)('onRightClick', this.polygon).bind(this),
        onHide: (0, _createEventCallback2.default)('onHide', this.polygon).bind(this),
        onShow: (0, _createEventCallback2.default)('onShow', this.polygon).bind(this),
        onMouseDown: (0, _createEventCallback2.default)('onMouseDown', this.polygon).bind(this),
        onMouseUp: (0, _createEventCallback2.default)('onMouseUp', this.polygon).bind(this),
        onMouseOver: (0, _createEventCallback2.default)('onMouseOver', this.polygon).bind(this),
        onMouseOut: (0, _createEventCallback2.default)('onMouseOut', this.polygon).bind(this),
        onChange: (0, _createEventCallback2.default)('onChange', this.polygon).bind(this),
        onTouchStart: (0, _createEventCallback2.default)('onTouchStart', this.polygon).bind(this),
        onTouchMove: (0, _createEventCallback2.default)('onTouchMove', this.polygon).bind(this),
        onTouchEnd: (0, _createEventCallback2.default)('onTouchEnd', this.polygon).bind(this)
      };
    }

    /**
     * Hide or show polygon.
     */

  }, {
    key: 'toggleVisible',
    value: function toggleVisible(previousProp, nextProp) {
      if (!(0, _isShallowEqual2.default)(previousProp, nextProp)) {
        if (nextProp === true) this.polygon.show();
        if (nextProp === false) this.polygon.hide();
      }
    }

    /**
     * Update AMap.Polygon instance with named API and given value.
     * Won't call API if the given value does not change.
     */

  }, {
    key: 'updatePolygonWithApi',
    value: function updatePolygonWithApi(apiName, previousProp, nextProp, newProp) {
      if (!(0, _isShallowEqual2.default)(previousProp, nextProp)) {
        this.polygon[apiName](newProp);
      }
    }

    /**
     * Render nothing.
     */

  }, {
    key: 'render',
    value: function render() {
      return null;
    }
  }]);

  return Polygon;
}(_react2.default.Component), _class.propTypes = {
  /**
   * Show polygon by default, you can toggle show or hide by changing visible.
   */
  visible: _propTypes2.default.bool,
  /**
   * Event callback.
   * Signature:
   * (polygon, ...event) => void
   * polygon: AMap.Polygon instance.
   * event: AMap event.
   */
  /* eslint-disable react/sort-prop-types,react/no-unused-prop-types */
  onComplete: _propTypes2.default.func,
  onClick: _propTypes2.default.func,
  onDblClick: _propTypes2.default.func,
  onRightClick: _propTypes2.default.func,
  onHide: _propTypes2.default.func,
  onShow: _propTypes2.default.func,
  onMouseDown: _propTypes2.default.func,
  onMouseUp: _propTypes2.default.func,
  onMouseOver: _propTypes2.default.func,
  onMouseOut: _propTypes2.default.func,
  onChange: _propTypes2.default.func,
  onTouchStart: _propTypes2.default.func,
  onTouchMove: _propTypes2.default.func,
  onTouchEnd: _propTypes2.default.func
  /* eslint-enable */
}, _class.defaultProps = {
  visible: true
}, _class.contextType = _AMapContext2.default, _temp);
exports.default = Polygon;