'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

/**
 * Deep copy.
 * The primitive value returns original value.
 * Reference values:
 * If options is array, deep copy all.
 * If options is object, the corresponding field will be copied in depth.
 * But if needCloneKeys is undefined, will all be copy.
 * @param  {node}  options
 * @param  {undefined|Array} needCloneKeys - Config the keys for the required copy fields
 * @return {node}
 */


exports.default = function (options, needCloneKeys) {
  if (options === null) {
    return null;
  }

  if ((typeof options === 'undefined' ? 'undefined' : _typeof(options)) !== 'object') {
    return options;
  }

  if (options instanceof Array) {
    return (0, _cloneDeep2.default)(options);
  }

  if (options instanceof Object) {
    if (needCloneKeys === void 0) {
      return (0, _cloneDeep2.default)(options);
    }

    return _extends({}, options, needCloneKeys.reduce(function (result, key) {
      if (!Object.keys(options).includes(key)) {
        return result;
      }

      return _extends({}, result, _defineProperty({}, key, (0, _cloneDeep2.default)(options[key])));
    }, {}));
  }

  return options;
};

var _cloneDeep = require('lodash/cloneDeep');

var _cloneDeep2 = _interopRequireDefault(_cloneDeep);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }